// ChildView.cpp

#include "stdafx.h"
#include "QEnvelopeTextDemo.h"
#include "ChildView.h"
#include "QEnvelopeText.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

const int nEnvelopes = 5;

// Traditional dummy text. See: http://www.lipsum.com/.
LPCTSTR sLorem = _T("Lorem ipsum dolor sit amet, consectetuer adipiscing elit, ")
	_T("sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. ")
	_T("Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. ")
	_T("Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, ")
	_T("vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit ")
	_T("praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi. ");

// Bzier data for test path
Point pntPallette[] =
{
	Point(80, 615),
	Point(80, 851), Point(160, 963), Point(517, 937),
	Point(703,917), Point(954, 699), Point(900, 640),
	Point(833, 570), Point(780, 696), Point(727, 696),
	Point(700, 696), Point(671, 669), Point(770, 538),
	Point(858, 411), Point(939, 408), Point(939, 266),
	Point(939, 126), Point(850, 74), Point(679, 74),
	Point(408, 74), Point(80, 386), Point(80, 615)
};

Point pntHole[] =
{
	Point(525, 678),
	Point(539, 697), Point(582, 694), Point(620, 640),
	Point(655, 594), Point(673, 534), Point(660, 517),
	Point(644, 496), Point(598, 488), Point(550, 556),
	Point(500, 623), Point(513, 662), Point(525, 678)
};


// CChildView

CChildView::CChildView()
: m_bShowPath(false)
, m_ScaleMin(90)
, m_ScaleMax(110)
, m_Justification(QBlockText::justFull)
, m_ReturnsInText(false)
, m_colPath(RGB(160, 160, 160))
, m_colText(RGB(0, 64, 128))
, m_iEnvelope(0)
{
	const int PointSize = 12;

	HDC hDC = ::GetDC(NULL);
	::ZeroMemory(& m_LogFont, sizeof(LOGFONT));
	m_LogFont.lfHeight = -::MulDiv(PointSize, ::GetDeviceCaps(hDC, LOGPIXELSY), 72);
	m_LogFont.lfWeight = FW_NORMAL;
	m_LogFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
	m_LogFont.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	m_LogFont.lfQuality = PROOF_QUALITY;
	m_LogFont.lfPitchAndFamily = FF_ROMAN;
	_tcscpy(m_LogFont.lfFaceName, _T("Georgia"));
	::ReleaseDC(NULL, hDC);
}

CChildView::~CChildView()
{
}

BEGIN_MESSAGE_MAP(CChildView, CWnd)
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_COMMAND(ID_OPTIONS_FONT, OnOptionsFont)
	ON_COMMAND(ID_OPTIONS_SHOWPATH, OnOptionsShowpath)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_SHOWPATH, OnUpdateOptionsShowpath)
	ON_COMMAND(ID_OPTIONS_SCALEMARGINS, OnOptionsScaleMargins)
	ON_COMMAND(ID_OPTIONS_PATHCOLOR, OnOptionsPathColor)
	ON_COMMAND(ID_OPTIONS_TEXTCOLOR, OnOptionsTextColor)
	ON_COMMAND_RANGE(ID_ENVELOPE_ENV00, ID_ENVELOPE_ENV11, OnEnvelope)
	ON_UPDATE_COMMAND_UI_RANGE(ID_ENVELOPE_ENV00, ID_ENVELOPE_ENV11, OnUpdateEnvelope)
	ON_COMMAND(ID_ENVELOPE_PREV, OnEnvelopePrev)
	ON_COMMAND(ID_ENVELOPE_NEXT, OnEnvelopeNext)
	ON_COMMAND(ID_OPTIONS_RETURNSINTEXT, OnOptionsReturns)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_RETURNSINTEXT, OnUpdateOptionsReturns)
	ON_COMMAND_RANGE(ID_JUSTIFICATION_LEFT, ID_JUSTIFICATION_HARD, OnJustification)
	ON_UPDATE_COMMAND_UI_RANGE(ID_JUSTIFICATION_LEFT, ID_JUSTIFICATION_HARD, OnUpdateJustification)
END_MESSAGE_MAP()

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), reinterpret_cast<HBRUSH>(COLOR_WINDOW+1), NULL);

	return TRUE;
}

void CChildView::OnPaint() 
{
	const REAL penWidth = 2.0f;
	const REAL dotSize = 6.0f;

	CPaintDC dc(this);

	Graphics g(dc.GetSafeHdc());
	g.SetSmoothingMode(SmoothingModeHighQuality);

	if (m_bShowPath)
	{
		Color colPath;
		colPath.SetFromCOLORREF(m_colPath);
		Pen pen(colPath, penWidth);
		pen.SetDashStyle(DashStyleDot);
		g.DrawPath(& pen, & m_EnvelopePath);
	}

	Color colText;
	colText.SetFromCOLORREF(m_colText);
	SolidBrush brush(colText);
	g.FillPath(& brush, & m_TextPath);
}

void CChildView::OnSize(UINT nType, int cx, int cy)
{
	CWnd::OnSize(nType, cx, cy);

	if (cx != 0 && cy != 0) RefreshPath();
}

void CChildView::OnEnvelopePrev()
{
	--m_iEnvelope;
	if (m_iEnvelope < 0) m_iEnvelope = nEnvelopes - 1;
	RefreshPath();
	Invalidate();
}

void CChildView::OnEnvelopeNext()
{
	++m_iEnvelope;
	if (m_iEnvelope >= nEnvelopes) m_iEnvelope = 0;
	RefreshPath();
	Invalidate();
}

void CChildView::OnEnvelope(UINT nID)
{
	m_iEnvelope = nID - ID_ENVELOPE_ENV00;
	RefreshPath();
	Invalidate();
}

void CChildView::OnUpdateEnvelope(CCmdUI *pCmdUI)
{
	int id = pCmdUI->m_nID - ID_ENVELOPE_ENV00;
	pCmdUI->SetCheck(id == m_iEnvelope ? 1 : 0);
}

void CChildView::OnOptionsFont()
{
	CFontDialog dlg(& m_LogFont, CF_SCALABLEONLY | CF_NOVECTORFONTS | CF_INITTOLOGFONTSTRUCT | CF_NOVERTFONTS | CF_SCREENFONTS);
	if (dlg.DoModal() == IDOK)
	{
		dlg.GetCurrentFont(& m_LogFont);
		RefreshPath();
		Invalidate();
	}
}

void CChildView::OnOptionsScaleMargins()
{
	CScaleMarginsDlg dlg(this);
	dlg.m_Min = m_ScaleMin;
	dlg.m_Max = m_ScaleMax;
	if (dlg.DoModal() == IDOK)
	{
		m_ScaleMin = dlg.m_Min;
		m_ScaleMax = dlg.m_Max;
		RefreshPath();
		Invalidate();
	}
}

void CChildView::OnJustification(UINT nID)
{
	m_Justification = nID - ID_JUSTIFICATION_LEFT;
	RefreshPath();
	Invalidate();
}

void CChildView::OnUpdateJustification(CCmdUI *pCmdUI)
{
	UINT id = pCmdUI->m_nID - ID_JUSTIFICATION_LEFT;
	pCmdUI->SetCheck(id == m_Justification);
}

void CChildView::OnOptionsReturns()
{
	m_ReturnsInText ^= true;
	RefreshPath();
	Invalidate();
}

void CChildView::OnUpdateOptionsReturns(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck(m_ReturnsInText ? 1 : 0);
}

void CChildView::OnOptionsShowpath()
{
	m_bShowPath ^= true;
	Invalidate();
}

void CChildView::OnUpdateOptionsShowpath(CCmdUI *pCmdUI)
{
	pCmdUI->SetCheck(m_bShowPath ? 1 : 0);
}

void CChildView::OnOptionsPathColor()
{
	CColorDialog dlg(m_colPath, CC_FULLOPEN | CC_SOLIDCOLOR);
	if (dlg.DoModal() == IDOK)
	{
		m_colPath = dlg.GetColor();
		if (m_bShowPath) Invalidate();
	}
}

void CChildView::OnOptionsTextColor()
{
	CColorDialog dlg(m_colText, CC_FULLOPEN | CC_SOLIDCOLOR);
	if (dlg.DoModal() == IDOK)
	{
		m_colText = dlg.GetColor();
		Invalidate();
	}
}

void CChildView::RefreshPath()
{
	CWaitCursor wait;

	m_TextPath.Reset();
	m_EnvelopePath.Reset();

	CRect rcClient;
	GetClientRect(rcClient);
	int w = rcClient.Width();
	int h = rcClient.Height();
	int dx = w / 10;
	int dy = h / 10;

	// Create a test string.
	CString s;
	for (int i = 0; i < 10; ++i)	// Ten 'lorems' must be enough.
	{
		s += sLorem;
		if (m_ReturnsInText) s += _T("\r\n");
	}

	switch (m_iEnvelope)
	{
	case 0:		// Pallette
		{
			// Fill test path with a figure.
			m_EnvelopePath.AddBeziers(pntPallette, sizeof(pntPallette) / sizeof(Point));
			m_EnvelopePath.CloseFigure();
			m_EnvelopePath.AddBeziers(pntHole, sizeof(pntHole) / sizeof(Point));
			m_EnvelopePath.CloseFigure();

			// Figure is designed for 1000x1000 window; scale to make it fit in client rect.
			Matrix mat;
			mat.Scale((REAL) rcClient.Width() / 1000.0f, (REAL) rcClient.Height() / 1000.0f);
			m_EnvelopePath.Transform(& mat);
		}
		break;
	case 1:		// Ellipse
		m_EnvelopePath.AddEllipse(dx, dy, w - 2 * dx, h - 2 * dy);
		break;
	case 2:		// Cross (two open paths)
		m_EnvelopePath.AddLine(dx, dy, w - dx, h - dy);
		m_EnvelopePath.CloseFigure();
		m_EnvelopePath.AddLine(w - dx, dy, dx, h - dy);
		m_EnvelopePath.CloseFigure();
		break;
	case 3:		// Q?
		{
			FontFamily ff(L"Georgia");
			m_EnvelopePath.AddString(L"Q?", -1, &ff, FontStyleBold, 40.0f, Point(0, 0), NULL);
			RectF rcBounds;
			m_EnvelopePath.GetBounds(& rcBounds);

			Matrix mat;
			mat.Translate(- rcBounds.GetLeft(), - rcBounds.GetTop());
			mat.Scale((REAL)(w - 2 * dx) / rcBounds.Width, (REAL)(h - 2 * dy) / rcBounds.Height, MatrixOrderAppend);
			mat.Translate((REAL) dx, (REAL) dy, MatrixOrderAppend);
			m_EnvelopePath.Transform(& mat);
		}
		break;
	case 4:		// QBlockText
		{
			// Create two rectangles.
			int gutter = dx / 4;
			REAL wRC = (REAL)(w - 2 * dx - gutter) / 2;
			REAL hRC = (REAL)(h - 2 * dy);
			RectF rc1((REAL) dx, (REAL) dy, wRC, hRC);
			RectF rc2((REAL) dx + gutter + wRC, (REAL) dy, wRC, hRC);

			// Put them in m_EnvelopePath, purely for showing path, not
			// for formatting text.
			m_EnvelopePath.AddRectangle(rc1);
			m_EnvelopePath.AddRectangle(rc2);

			// Set up QBlockText
			QBlockText bt;
			bt.SetScaleMargins((REAL) m_ScaleMin / 100.0f, (REAL) m_ScaleMax / 100.0f);
			bt.SetJustification((QBlockText::justification) m_Justification);
			bt.SetFont(m_LogFont);

			// Retrieve text path for rc1; return value is number of processed characters.
			int r = bt.GetBlockText(m_TextPath, s, rc1);

			// Add text path for rc2, starting at first unprocessed character.
			r = bt.GetBlockText(m_TextPath, (LPCTSTR) s + r, rc2);

			TRACE2(_T("String length = %d, QBlockText::GetBlockText() returns %d\n"), s.GetLength(), r);

			// Exit, we don't need QEnvelopeText for this test.
			return;
		}
		break;
	default:
		break;
	}

	// Set up QEnvelopeText.
	QEnvelopeText et(m_EnvelopePath);
	et.SetScaleMargins((REAL) m_ScaleMin / 100.0f, (REAL) m_ScaleMax / 100.0f);
	et.SetJustification((QBlockText::justification) m_Justification);
	et.SetFont(m_LogFont);

	// Retrieve text path.
	int nProcessed = et.GetEnvelopeText(m_TextPath, s);

	TRACE2(_T("String length = %d, QEnvelopeText::GetEnvelopeText() returns %d\n"), s.GetLength(), nProcessed);
}


// CScaleMarginsDlg dialog

IMPLEMENT_DYNAMIC(CScaleMarginsDlg, CDialog)
CScaleMarginsDlg::CScaleMarginsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CScaleMarginsDlg::IDD, pParent)
	, m_Min(0)
	, m_Max(0)
{
}

CScaleMarginsDlg::~CScaleMarginsDlg()
{
}

void CScaleMarginsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_MIN, m_Min);
	DDV_MinMaxInt(pDX, m_Min, 20, 100);
	DDX_Text(pDX, IDC_MAX, m_Max);
	DDV_MinMaxInt(pDX, m_Max, 100, 1000);
	DDX_Control(pDX, IDCS_MIN, m_spinMin);
	DDX_Control(pDX, IDCS_MAX, m_spinMax);
}

BEGIN_MESSAGE_MAP(CScaleMarginsDlg, CDialog)
END_MESSAGE_MAP()

BOOL CScaleMarginsDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	m_spinMin.SetRange(20, 100);
	m_spinMax.SetRange(100, 1000);
	return TRUE;
}
